// Oldham, Jeffrey D.
// 2000Apr22
// CS3352

// Checking Scanned Ibbotsson Data for Self-Consistency
// Income Data

// command-line arguments:
// 1. income-data file
// 2. capital-appreciation data file
// 3. desired accuracy

// ASSUME no lines begin with #.

#include <iostream>
#include <fstream>
#include <stdlib.h>		// has EXIT_SUCCESS
#include <vector>
#include <algorithm>
#include <iterator>
#include <math.h>


class dataLine {
public:
  dataLine(void) { v.resize(12); }
  friend istream& operator>>(istream & in, dataLine & d) {
    if (in >> d.year) {
      for (int i = 0; i < 12; ++i)
	if (!(in >> d.v[i]))
	  return in;
      if (in >> d.year2 >> d.total)
	return in;
    }
    return in;
  }
  unsigned long year, year2;
  vector<double> v;
  double total;

  bool checkInternalConsistency(void) {
    return year == year2;
  }
};


bool checkIncome(const dataLine & income, const dataLine & capAppr,
		 const double accur) {
  double currentValue = 1.0;
  double totalIncome = 0.0;
  for (int i = 0; i < 12; ++i) {
    totalIncome += currentValue * income.v[i];
    currentValue *= (1.0 + capAppr.v[i]);
  }
  return fabs(totalIncome - income.total) <= accur;
}


int main(int argc, char *argv[])
{
  cout << "Hello, world.\n";
  if (argc != 4) {
    cerr << argv[0] << ": income-data-file cap-appr-data-file desired-accuracy\n";
    throw "illegal command-line";
  }

  ifstream incomeData(argv[1]);
  if (!incomeData) {
    cerr << argv[0] << ": bad income data file " << argv[1] << endl;
    throw "bad income data file";
  }
  ifstream capApprData(argv[2]);
  if (!capApprData) {
    cerr << argv[0] << ": bad capital appreciation data file " <<
      argv[2] << endl;
    throw "bad capital appreciation data file";
  }
  double accur = strtod(argv[3], static_cast<char **>(0));

  // Read in a line at a time.
  dataLine incomeLine, capApprLine;
  while (incomeData >> incomeLine && capApprData >> capApprLine) {
  if (!incomeLine.checkInternalConsistency()) {
    cerr << "internal inconsistency for income year " <<
      incomeLine.year << endl;
    throw "bad data";
  }
  if (!capApprLine.checkInternalConsistency()) {
    cerr << "internal inconsistency for capAppr year " <<
      capApprLine.year << endl;
    throw "bad data";
  }
  if (capApprLine.year != incomeLine.year) {
    cerr << "different years for income (" << 
      incomeLine.year << ") and capital appreciation (" <<
      capApprLine.year << ")" << endl;
    throw "mismatched years";
  }
  if (!checkIncome(incomeLine, capApprLine, accur)) {
    cerr << "bad income line for " << incomeLine.year << endl;
    throw "bad income line";
  }
  cout << "finished checking " << incomeLine.year << endl;
  }

  incomeData.close();
  capApprData.close();
  cout << "Hello, world.\n";
  return EXIT_SUCCESS;
}
